use gio::glib::Object;
use gtk4 as gtk;

use std::str::FromStr;

use crate::{
    model::{models::SignalReduce, Error, Frequency, Result},
    model_utils::signal_reducer::*,
    view::components::input::Input,
};

use gtk::{
    prelude::{Cast, CastNone, EditableExt, ListItemExt, WidgetExt},
    ColumnView, Entry, Label, ListItem, SignalListItemFactory,
};

pub fn get_error_message(error: Error) -> Option<&'static str> {
    match error.to_string().as_str() {
        "cannot parse float from empty string" => Some("Вы не ввели данные в поле/-я"),
        "invalid float literal" => Some("Вы ввели не корректные данные в поле/-я"),
        _ => None,
    }
}

pub fn parse_fields(all_inputs: Vec<Input<Entry>>) -> Result<SignalReduce> {
    let mut values: [f64; 6] = [0.0; 6];

    for (i, input) in all_inputs.iter().enumerate() {
        let input_text_buffer = input.get_input();
        let extracted_value =
            f64::from_str(input_text_buffer.text().replace(',', ".").as_str().trim())?;
        values[i] = extracted_value;
    }

    Ok(SignalReduce {
        length: values[0],
        wire_resistance: values[1],
        wire_capacity: values[2],
        source_resistance: values[3],
        source_voltage: values[4],
        frequency: values[5],
    })
}

#[inline]
pub fn update_column_view(column_view: &ColumnView) {
    column_view.set_visible(false);
    column_view.set_visible(true);
}

#[inline]
pub fn column_view_setup_factory(_factory: &SignalListItemFactory, list_item: &Object) {
    list_item
        .downcast_ref::<ListItem>()
        .expect("Needs to be ListItem")
        .set_child(Some(&Label::new(None)));
}

pub fn get_cell_data(list_item: &Object) -> (Frequency, Label) {
    let cell_value = list_item
        .downcast_ref::<ListItem>()
        .expect("Needs to be ListItem")
        .item()
        .and_downcast::<Frequency>()
        .expect("The item has to be an `IntegerObject`.");

    let cell_label = list_item
        .downcast_ref::<ListItem>()
        .expect("Needs to be ListItem")
        .child()
        .and_downcast::<Label>()
        .expect("The child has to be a `Label`.");

    cell_label.set_selectable(true);

    (cell_value, cell_label)
}

pub fn frequency_column_fabric(list_item: &Object) {
    let (cell_value, cell_label) = get_cell_data(list_item);

    cell_label.set_label(&cell_value.frequency().to_string());
}

pub fn reactive_resist_fabric(list_item: &Object, values: SignalReduce) {
    let (cell_value, cell_label) = get_cell_data(list_item);

    cell_value.set_reactive_resist(reactive_resistance_of_capacitor(
        values.wire_capacity * 10f64.powi(-12),
        values.length,
        cell_value.frequency() * 10f64.powi(6),
    ));

    cell_label.set_label(format!("{0:.1$}", cell_value.reactive_resist(), 6).as_str());
}

pub fn signal_source_voltage_fabric(list_item: &Object, values: SignalReduce) {
    let (cell_value, cell_label) = get_cell_data(list_item);

    cell_value.set_full_resistance(full_resistance_of_capacitor(
        cell_value.reactive_resist(),
        values.source_resistance,
        values.wire_resistance,
        values.length,
    ));

    cell_value.set_signal_source_voltage(
        voltage_from_signal_source(
            values.source_voltage * 10f64.powi(-3),
            cell_value.reactive_resist(),
            cell_value.full_resistance(),
        ) * 1000.0,
    );

    cell_label.set_label(format!("{0:.1$}", cell_value.signal_source_voltage(), 6).as_str());
}

pub fn coef_fabric(list_item: &Object, values: SignalReduce) {
    let (cell_value, cell_label) = get_cell_data(list_item);

    let coef: f64 =
        coef_of_signal_reduce(values.source_voltage, cell_value.signal_source_voltage());

    cell_label.set_label(format!("{0:.1$}", coef, 6).as_str());
}
